//# sourceURL=widgets/Multibutton.js

import Widget from '../Widget.js';
import Entity from '/client/Entity.js';

var Multibutton = function( elem ) {
    Widget.call( this, elem );
    this.name = "Multibutton";
    this.rows = this.getData( "rows", 1 );
    this.cols = this.getData( "cols", 2 );
    this.labels = this.getData( "labels", false );
    $(this.elem).addClass("widget-multibutton");

    if ( this.labels ) {
        this.labels = this.labels.split( / *, */ );
    }

    this.log.setLevel( 8 );
};

Multibutton.prototype = Object.create(Widget.prototype);
Multibutton.prototype.constructor = Multibutton;

Multibutton.prototype.layoutInit = function( layoutName, descObj ) {
    var $grp = $( 'div.buttongroup', this.elem );
    console.log("width",this.elem.innerWidth(),"height",this.elem.innerHeight());
    var bwidth = Math.floor( (this.elem.innerWidth() - 12*(this.cols-1)) / this.cols );
    var bheight = Math.floor( (this.elem.innerHeight() - 12*(this.rows-1)) / this.rows );
    $( 'button', $grp ).remove();
    var $btn = $( '<button></button>' ).css( { width: bwidth, height: bheight, "font-size": Math.floor( bheight/4 ) } );
    var ix = 0;
    for ( var row=0; row<this.rows; ++row ) {
        var $row = $( '<div></div>' ).appendTo( $grp );
        for ( var col=0; col<this.cols; ++col ) {
            var $b = $btn.clone();
            if ( this.labels && ix < this.labels.length ) {
                $b.text( this.labels[ix] );
            } else {
                $b.text( String(ix) );
            }
            $b.attr( 'id', String(self.id) + ix );
            $b.appendTo( $row );
            ix++;
        }
    }
};

Multibutton.prototype.post_update = function( event ) {
    console.log("post_update",event);
};

Multibutton.prototype.click = function( event ) {
    var $btn = $( event.target );
    if ( $btn.is( 'button' ) ) {
        var id = parseInt( $btn.attr( 'id' ).replace( self.id, "" ).replace( /^-/, "" ) );
        ++id;
        console.log(String(this),"calling action for button",id);
        var act = this.getData( "action" + id, "" );
        if ( "" !== act ) {
            /* Form is [entity-selection;]action[(param=value [,...])]    */
            var e;
            var i = act.indexOf( ';' );
            if ( i < 0 ) {
                e = this.entity;
            } else {
                var e_expr = act.substring( 0, i );
                var act = act.substr( i+1 );
                console.log("expr",e_expr,"act",act);
                var e;
                if ( "" === e_expr || "entity" === e_expr ) {
                    e = this.entity;
                } else {
                    e = SlapDash.findEntity( e_expr );
                }
            }
            if ( e instanceof Entity ) {
                var params = {};
                i = act.indexOf( '(' );
                if ( i > 0 ) {
                    var p = act.slice( i+1, -1 );
                    act = act.substring( 0, i );
                    var t = p.split( / *, */ );
                    t.forEach( function( tt ) {
                        var m = tt.match( /^([^=]+)=(.*)/ );
                        params[m[1]] = m[2];
                    });
                }
                this.log.warn("%1 perform %2 on %4 with %3", this, act, params, e);
                e.perform( act, params );
            } else {
                this.log.warn("Entity expression %1 did not resolve to an entity %2", e_expr, e);
                console.log(e);
            }
            return false;
        } else {
            this.log.warn( "%1 no action defined for button %2", this, id);
        }
    }
    return true;
};

export default Multibutton;
